package ru.yarsu

import org.http4k.core.*
import ru.yarsu.formats.JacksonMessage
import ru.yarsu.formats.jacksonMessageLens
import ru.yarsu.models.PebbleViewModel
import org.http4k.core.Method.GET
import org.http4k.core.Status.Companion.OK
import org.http4k.filter.DebuggingFilters.PrintRequest
import org.http4k.routing.ResourceLoader
import org.http4k.routing.bind
import org.http4k.routing.routes
import org.http4k.routing.static
import org.http4k.server.Netty
import org.http4k.server.asServer
import org.http4k.template.PebbleTemplates
import org.http4k.template.TemplateRenderer

private fun replyToKotest(): HttpHandler = { request ->
    Response(OK).body("Echo '${request.bodyString()}'")
}

private fun showPebbleTemplate(renderer: TemplateRenderer): HttpHandler = {
    val viewModel = PebbleViewModel("Hello there!")
    Response(OK).body(renderer(viewModel))
}

private fun respondWithJson(): (Request) -> Response = {
    Response(OK).with(jacksonMessageLens of JacksonMessage("Barry", "Hello there!"))
}

private fun respondeWithPong(): HttpHandler = {
    Response(OK).body("pong")
}

fun router(renderer: TemplateRenderer): HttpHandler = routes(
    "/ping" bind GET to respondeWithPong(),
    "/formats/json/jackson" bind GET to respondWithJson(),
    "/templates/pebble" bind GET to showPebbleTemplate(renderer),
    "/testing/kotest" bind GET to replyToKotest(),
    static(ResourceLoader.Classpath("/ru/yarsu/public/")),
)

fun main() {
    val renderer = PebbleTemplates().HotReload("src/main/resources")
    val app = router(renderer)
    val printingApp: HttpHandler = PrintRequest().then(app)
    val server = printingApp.asServer(Netty(9000)).start()
    println("Server started on http://localhost:" + server.port())
}
